<?php
/**
 * ownCloud Admin_Audit
 *
 * @author Frank Karlitscheck <frank@owncloud.com>
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @author Tom Needham <tom@owncloud.com>
 * @author Michael Gapczynski
 * @author Thomas Müller <deepdiver@owncloud.com>
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright (C) 2016 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Admin_Audit\Handlers;

use OCP\Constants;
use OCP\Files\Node;
use OCP\Share;
use Symfony\Component\EventDispatcher\GenericEvent;

class Sharing extends Base {

	static public function unshareInfo($arguments) {
		self::logUnshareInfo('{actor} unshare {targetmount} from {name}', $arguments);
	}

	static public function federatedShareArrivedFromRemote($arguments) {
		self::logFederatedShareArriveFromRemote('{actor} received federated share {name} from {targetuser}', $arguments);
	}

	static public function decline_remote_shared($arguments) {
		self::logDeclineShareHook('{actor} declined share {type} from {targetuser}', $arguments);
	}

	static public function accept_remote_shared($arguments) {
		self::logAcceptShareHook('{actor} accepted share {type} from {targetuser}', $arguments);
	}

	static protected function logUnshareInfo($message, $arguments) {
		self::getLogger()->log($message, [
			'name' => $arguments['user'],
			'targetmount' => $arguments['targetmount']
		]);
	}
	static protected function logFederatedShareArriveFromRemote($message, $arguments) {
		self::getLogger()->log($message, [
			'name' => $arguments['name'],
			'targetuser' => $arguments['targetuser']
		]);
	}

	static protected function logDeclineShareHook($message, $arguments) {
		self::getLogger()->log($message,[
			'type' => $arguments['sharedItem'],
			'targetuser' => $arguments['shareAcceptedFrom'] . "@" . $arguments['remoteUrl']
		]);
	}

	static protected function logAcceptShareHook($message, $arguments) {
		self::getLogger()->log($message,[
			'type' => $arguments['sharedItem'],
			'targetuser' => $arguments['shareAcceptedFrom'] . "@" . $arguments['remoteUrl']
		]);
	}

	static public function post_shared($arguments) {
		self::logShareHook('{actor} shared {type} {path} with {target}, permissions: {permissions}, owner: {owner}', $arguments);
	}

	static protected function logShareHook($message, array $arguments) {
		if ($arguments['itemType'] !== 'file' && $arguments['itemType'] !== 'folder') {
			return;
		}

		$nodes = \OC::$server->getUserFolder($arguments['uidOwner'])->getById($arguments['itemSource']);

		if (empty($nodes)) {
			return;
		}
		/** @var Node $node */
		$node = array_shift($nodes);

		if ($arguments['shareType'] === Share::SHARE_TYPE_LINK) {
			$target = 'a public link';
			if (isset($arguments['token'])) {
				$target .= ', token: ' . $arguments['token'];
			}
			if (!empty($arguments['shareWith']) or ($arguments['passwordEnabled'])) {
				$target .= ', with password';
			} else {
				$target .= ', without password';
			}
		} else if($arguments['shareType'] === Share::SHARE_TYPE_GROUP) {
			$target = 'the group ' . $arguments['shareWith'];
		} else if($arguments['shareType'] === Share::SHARE_TYPE_USER) {
			$target = 'the user ' . $arguments['shareWith'];
		} else if ($arguments['shareType'] === Share::SHARE_TYPE_REMOTE) {
			$target = 'the remote user ' . $arguments['shareWith'];
		} else {
			return;
		}

		self::getLogger()->log($message, [
			'type' => $arguments['itemType'],
			'path' => $node->getPath(),
			'target' => $target,
			'permissions' => self::getPermissionsString($arguments['permissions']),
			'oldpermissions' => isset($arguments['oldpermissions']) ? self::getPermissionsString($arguments['oldpermissions']) : '',
			'owner' => $node->getOwner()->getUID(),
		]);
	}

	static public function post_unshare($arguments) {
		if ($arguments['itemType'] !== 'file' && $arguments['itemType'] !== 'folder') {
			return;
		}

		$nodes = \OC::$server->getUserFolder($arguments['uidOwner'])->getById($arguments['itemSource']);

		if (empty($nodes)) {
			return;
		}
		/** @var Node $node */
		$node = array_shift($nodes);

		if ($arguments['shareType'] === Share::SHARE_TYPE_LINK) {
			self::getLogger()->log('{actor} removed the share link for {type} {path}, owner: {owner}', [
				'type' => $arguments['itemType'],
				'path' => $node->getPath(),
				'owner' => $node->getOwner()->getUID(),
			]);
			return;
		} else if($arguments['shareType'] === Share::SHARE_TYPE_GROUP) {
			$target = 'group ' . $arguments['shareWith'];
		} else if($arguments['shareType'] === Share::SHARE_TYPE_USER) {
			$target = 'user ' . $arguments['shareWith'];
		} else if ($arguments['shareType'] === Share::SHARE_TYPE_REMOTE) {
			$target = 'remote user ' . $arguments['shareWith'];
		} else {
			return;
		}

		self::getLogger()->log('{actor} unshared {type} {path} with remote user {target}, owner: {owner}', [
			'type' => $arguments['itemType'],
			'path' => $node->getPath(),
			'target' => $target,
			'owner' => $node->getOwner()->getUID(),
		]);
	}

	static public function unshareFromRecipient($arguments) {
		self::getLogger()->log('{actor} unshared {type} {path} from user {owner}, owner path: {ownerPath}',[
			'type' => $arguments['nodeType'],
			'path' => $arguments['recipientPath'],
			'owner' => $arguments['shareOwner'],
			'ownerPath' => $arguments['ownerPath']
		]);
	}

	public static function shareUpdate(GenericEvent $arguments) {
		$share = $arguments->getArgument('shareobject');
		$itemType = $share->getNode() instanceof \OCP\Files\File ? 'file' : 'folder';
		$itemSource = $share->getNode()->getId();
		$shareType = $share->getShareType();
		$shareWith = $share->getSharedWith();
		$permissions = $share->getPermissions();
		$uidOwner = $share->getSharedBy();

		$nodes = \OC::$server->getUserFolder($uidOwner)->getById($itemSource);

		if (empty($nodes)) {
			return;
		}
		/** @var Node $node */
		$node = array_shift($nodes);

		if ($arguments->hasArgument('permissionupdate') &&($arguments->getArgument('permissionupdate') === true)) {
			$eventData = [
				'itemType' => $itemType,
				'itemSource' => $itemSource,
				'shareType' => $shareType,
				'shareWith' => $shareWith,
				'uidOwner' => $uidOwner,
				'permissions' => $permissions,
				'oldpermissions' => $arguments->getArgument('oldpermissions'),
				'path' => $arguments->getArgument('path')
			];
			self::logShareHook('{actor} updated the permissions for {type} {path} for {target}, from old permissions: {oldpermissions} to new permissions: {permissions}, owner: {owner}', $eventData);
		}
		if ($arguments->hasArgument('passwordupdate') && ($arguments->getArgument('passwordupdate') === true)) {
			$disabled = is_null($share->getPassword());
			$action = $disabled ? 'removed password from' : 'updated password on';
			self::getLogger()->log('{actor} ' . $action . ' link-shared {type} {path}, token: {token}, owner: {owner}', [
				'token' => $share->getToken(),
				'type' => $itemType,
				'path' => $node->getPath(),
				'owner' => $node->getOwner()->getUID(),
			]);
		}
		if ($arguments->hasArgument('expirationdateupdated') && ($arguments->getArgument('expirationdateupdated') === true)) {
			$date = $share->getExpirationDate();
			if ($date instanceof \DateTime) {
				$date = $date->format('Y-m-d');
			}
			$olddate = $arguments->getArgument('oldexpirationdate');
			if ($olddate instanceof \DateTime) {
				$olddate = $olddate->format('Y-m-d');
			}

			if (is_null($date)) {
				self::getLogger()->log('{actor} removed expiration date from shared {type} {path}, owner: {owner}', [
					'type' => $itemType,
					'path' => $node->getPath(),
					'owner' => $node->getOwner()->getUID(),
				]);
			} else {
				if (is_null($olddate)) {
					self::getLogger()->log('{actor} set expiration date {date} on shared {type} {path}, owner: {owner}', [
						'date' => $date,
						'type' => $itemType,
						'path' => $node->getPath(),
						'owner' => $node->getOwner()->getUID(),
					]);
				} else {
					self::getLogger()->log('{actor} update expiration from date {olddate} to date {date} on shared {type} {path}, owner: {owner}', [
						'date' => $date,
						'olddate' => $olddate,
						'type' => $itemType,
						'path' => $node->getPath(),
						'owner' => $node->getOwner()->getUID(),
						]);
				}
			}
		}
		if ($arguments->hasArgument('sharenameupdated') && ($arguments->getArgument('sharenameupdated') === true)) {
			self::getLogger()->log('{actor} share name updated from "{oldname}" to "{newname}" for shared {type} {path}, owner: {owner}', [
				'oldname' => $arguments->getArgument('oldname'),
				'newname' => $share->getName(),
				'type' => $itemType,
				'path' => $node->getPath(),
				'owner' => $node->getOwner()->getUID()
			]);
		}
	}

	public static function share_link_access($arguments) {
		if ($arguments['itemType'] !== 'file' && $arguments['itemType'] !== 'folder' && $arguments['itemType'] !== '') {
			return;
		}

		if($arguments['errorCode'] === 200) {
			$result = 'success.';
		} else {
			$result = 'error: ' . $arguments['errorMessage'];
		}

		$detailInfo = $path = $owner = '';
		if(!empty($arguments['itemType'])) {
			$detailInfo = 'd {type} {path}, owner: {owner},';
			$nodes = \OC::$server->getUserFolder($arguments['uidOwner'])->getById($arguments['itemSource']);

			if (empty($nodes)) {
				return;
			}
			/** @var Node $node */
			$node = array_shift($nodes);

			$path = $node->getPath();
			$owner = $node->getOwner()->getUID();
		}
		self::getLogger()->log('Share' . $detailInfo . ' with token {token} was accessed with ' . $result, [
			'token' => $arguments['token'],
			'type' => $arguments['itemType'],
			'path' => $path,
			'owner' => $owner,
		]);
	}

	static protected function getPermissionsString($given) {
		$permissions = [];
		if ($given & Constants::PERMISSION_READ) {
			$permissions[] = ' READ';
		}
		if ($given & Constants::PERMISSION_UPDATE) {
			$permissions[] = ' UPDATE';
		}
		if ($given & Constants::PERMISSION_CREATE) {
			$permissions[] = ' CREATE';
		}
		if ($given & Constants::PERMISSION_DELETE) {
			$permissions[] = ' DELETE';
		}
		if ($given & Constants::PERMISSION_SHARE) {
			$permissions[] = ' SHARE';
		}

		return implode(' ', $permissions);
	}
}
