<?php
/**
 *
 * @author Joas Schilling <nickvergessen@owncloud.com>
 * @copyright 2016 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Admin_Audit\Handlers;


use OCA\Admin_Audit\Logger;
use OCP\Files\Config\ICachedMountInfo;
use OCP\Files\Config\IMountProviderCollection;
use OCP\Files\IRootFolder;
use OCP\Files\Node;
use OCP\Share;
use OCP\SystemTag\ISystemTagManager;
use OCP\SystemTag\ManagerEvent;
use OCP\SystemTag\MapperEvent;
use OCP\SystemTag\TagNotFoundException;

class SystemTags {
	/** @var \OCP\SystemTag\ISystemTagManager */
	protected $tagManager;
	/** @var \OCP\Files\Config\IMountProviderCollection */
	protected $mountCollection;
	/** @var \OCP\Files\IRootFolder */
	protected $rootFolder;
	/** @var Logger */
	protected $logger;

	/**
	 * Listener constructor.
	 *
	 * @param ISystemTagManager $tagManager
	 * @param IMountProviderCollection $mountCollection
	 * @param IRootFolder $rootFolder
	 * @param Logger $logger
	 */
	public function __construct(ISystemTagManager $tagManager,
								IMountProviderCollection $mountCollection,
								IRootFolder $rootFolder,
								Logger $logger) {
		$this->tagManager = $tagManager;
		$this->mountCollection = $mountCollection;
		$this->rootFolder = $rootFolder;
		$this->logger = $logger;
	}

	/**
	 * @param ManagerEvent $event
	 */
	public function managerEvent(ManagerEvent $event) {
		if ($event->getEvent() === ManagerEvent::EVENT_CREATE) {
			$this->logger->log('{actor} created system tag "{tag}"', [
				'tag' => $this->logger->prepareTagAsParameter($event->getTag()),
			]);
		} else if ($event->getEvent() === ManagerEvent::EVENT_UPDATE) {
			$this->logger->log('{actor} updated system tag "{beforeTag}" to "{afterTag}"', [
				'afterTag' => $this->logger->prepareTagAsParameter($event->getTag()),
				'beforeTag' => $this->logger->prepareTagAsParameter($event->getTagBefore()),
			]);
		} else if ($event->getEvent() === ManagerEvent::EVENT_DELETE) {
			$this->logger->log('{actor} deleted system tag "{tag}"', [
				'tag' => $this->logger->prepareTagAsParameter($event->getTag()),
			]);
		}
	}

	/**
	 * @param MapperEvent $event
	 */
	public function mapperEvent(MapperEvent $event) {
		$tagIds = $event->getTags();
		if ($event->getObjectType() !== 'files' ||empty($tagIds)
			|| !in_array($event->getEvent(), [MapperEvent::EVENT_ASSIGN, MapperEvent::EVENT_UNASSIGN])) {
			// System tags not for files, no tags, not (un-)assigning
			return;
		}

		try {
			$tags = $this->tagManager->getTagsByIds($tagIds);
		} catch (TagNotFoundException $e) {
			// User assigned/unassigned a non-existing tag, ignore...
			return;
		}

		if (empty($tags)) {
			return;
		}

		// Get all mount point owners
		$cache = $this->mountCollection->getMountCache();
		$mounts = $cache->getMountsForFileId($event->getObjectId());
		if (!empty($mounts)) {
			/** @var ICachedMountInfo $mount */
			$mount = array_shift($mounts);

			$owner = $mount->getUser()->getUID();
			$ownerFolder = $this->rootFolder->getUserFolder($owner);
			$nodes = $ownerFolder->getById($event->getObjectId());

			if (!empty($nodes)) {
				/** @var Node $node */
				$node = array_shift($nodes);

				foreach ($tags as $tag) {
					if ($event->getEvent() === MapperEvent::EVENT_ASSIGN) {
						$logMessage = '{actor} assigned system tag "{tag}" to "{path}"{tags}';
						$tags = $this->logger->getTagStringForFile($event->getObjectId(), [], $tagIds);
					} else {
						$logMessage = '{actor} unassigned system tag "{tag}" from "{path}"{tags}';
						$tags = $this->logger->getTagStringForFile($event->getObjectId(), $tagIds);
					}

					$this->logger->log($logMessage, [
						'actor' => $this->getActor(),
						'path' => $node->getPath(),
						'tag' => $this->logger->prepareTagAsParameter($tag),
						'tags' => $tags,
					]);
				}
			}
		}
	}

	/**
	 * @return string
	 */
	protected function getActor() {
		$exception = new \Exception();

		foreach ($exception->getTrace() as $trace) {
			if (isset($trace['class']) && $trace['class'] === 'OCA\Workflow\AutoTagging\Tagger' &&
				isset($trace['function']) && $trace['function'] === 'executeRules'
			) {
				return 'Workflow Autotagger';
			}
		}

		return $this->logger->getUserOrIp();
	}
}
